//line parser.y:1

package main

import __yyfmt__ "fmt"

//line parser.y:4
import (
	"errors"
	"fmt"
	"strings"
	"unicode"
)

var program Prules = nil
var query Pterms = nil

//line parser.y:17
type TermSymType struct {
	yys   int
	name  string
	rule  *Prule
	rules Prules
	term  *Pterm
	terms Pterms
}

const NAME = 57346
const IMPL = 57347
const TERMOP = 57348

var TermToknames = [...]string{
	"$end",
	"error",
	"$unk",
	"NAME",
	"IMPL",
	"TERMOP",
	"'.'",
	"'('",
	"')'",
	"','",
}
var TermStatenames = [...]string{}

const TermEofCode = 1
const TermErrCode = 2
const TermMaxDepth = 200

//line parser.y:110

/* start of programs */

type TermLex struct {
	s   string
	pos int
}

func (l *TermLex) Lex(lval *TermSymType) int {
	s := l.s
	s = strings.TrimLeftFunc(s, unicode.IsSpace)
	if s == "" {
		return 0
	}
	for s[0] == '#' || s[0] == '%' {
		i := 1
		for s[i] != '\n' {
			i++
			if i >= len(s) {
				return 0
			}
		}
		s = s[i+1:]
		s = strings.TrimLeftFunc(s, unicode.IsSpace)
		if s == "" {
			return 0
		}
	}
	switch s[0] {
	case '(', ')', '.', ',':
		break
	case '/':
		fallthrough
	case '=':
		{
			l.s = s[1:]
			lval.name = string(s[0])
			return TERMOP
		}
	case ':':
		if s[1] == '-' {
			l.s = s[2:]
			return IMPL
		}
	case '"':
		if i := strings.Index(s[1:], `"`); i >= 0 {
			lval.name = string(s[1 : i+1])
			l.s = s[i+2:]
			return NAME
		}
	default:
		{
			i := 0
			r := rune(s[i])
			for (unicode.IsLetter(r) || unicode.IsNumber(r)) && !unicode.IsSpace(r) {
				i++
				if i >= len(s) {
					break
				}
				r = rune(s[i])
			}
			lval.name = string(s[:i])
			l.s = s[i:]
			return NAME
		}
	}
	l.s = s[1:]
	return int(s[0])
}

func (l *TermLex) Error(s string) {
	fmt.Printf("syntax error: %s %s\n", l.s, s)
}

func ClearParserResult() {
	program = nil
	query = nil
}

func ParseQuery(input string) (Pterms, error) {
	ClearParserResult()
	if TermParse(&TermLex{s: input}) != 0 || program != nil {
		return nil, errors.New("syntax error in Query")
	}
	return query, nil
}

func ParseTerm(input string) (*Pterm, error) {
	ClearParserResult()
	if TermParse(&TermLex{s: input + "."}) != 0 || query != nil {
		return nil, errors.New("syntax error in Term")
	}
	return program[0].Head, nil
}

func ParseRules(input string) (Prules, error) {
	ClearParserResult()
	if TermParse(&TermLex{s: input}) != 0 || query != nil {
		return nil, errors.New("syntax error in Rules")
	}
	return program, nil
}

//line yacctab:1
var TermExca = [...]int{
	-1, 1,
	1, -1,
	-2, 0,
}

const TermNprod = 14
const TermPrivate = 57344

var TermTokenNames []string
var TermStates []string

const TermLast = 29

var TermAct = [...]int{

	2, 6, 16, 4, 24, 9, 11, 23, 15, 9,
	9, 13, 18, 12, 19, 17, 20, 21, 22, 14,
	9, 8, 7, 8, 5, 1, 3, 0, 10,
}
var TermPact = [...]int{

	17, -1000, -1, 17, 6, -1000, 13, 19, 7, 19,
	-1000, 6, -1000, 19, 19, 10, -1000, 19, -1000, 0,
	-1000, -1000, -5, -1000, -1000,
}
var TermPgo = [...]int{

	0, 24, 26, 1, 2, 0, 25,
}
var TermR1 = [...]int{

	0, 6, 6, 2, 2, 1, 1, 1, 3, 3,
	5, 5, 4, 4,
}
var TermR2 = [...]int{

	0, 1, 1, 1, 2, 2, 4, 3, 4, 1,
	1, 3, 3, 1,
}
var TermChk = [...]int{

	-1000, -6, -5, -2, -4, -1, -3, 5, 4, 10,
	-1, -4, 7, 5, 6, -5, -4, 8, -4, -5,
	-3, 7, -5, 7, 9,
}
var TermDef = [...]int{

	0, -2, 1, 2, 10, 3, 13, 0, 9, 0,
	4, 0, 5, 0, 0, 0, 10, 0, 11, 0,
	12, 7, 0, 6, 8,
}
var TermTok1 = [...]int{

	1, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	8, 9, 3, 3, 10, 3, 7,
}
var TermTok2 = [...]int{

	2, 3, 4, 5, 6,
}
var TermTok3 = [...]int{
	0,
}

//line yaccpar:1

/*	parser for yacc output	*/

var (
	TermDebug        = 0
	TermErrorVerbose = false
)

type TermLexer interface {
	Lex(lval *TermSymType) int
	Error(s string)
}

type TermParser interface {
	Parse(TermLexer) int
	Lookahead() int
}

type TermParserImpl struct {
	lookahead func() int
	state     func() int
}

func (p *TermParserImpl) Lookahead() int {
	return p.lookahead()
}

func TermNewParser() TermParser {
	p := &TermParserImpl{
		lookahead: func() int { return -1 },
		state:     func() int { return -1 },
	}
	return p
}

const TermFlag = -1000

func TermTokname(c int) string {
	if c >= 1 && c-1 < len(TermToknames) {
		if TermToknames[c-1] != "" {
			return TermToknames[c-1]
		}
	}
	return __yyfmt__.Sprintf("tok-%v", c)
}

func TermStatname(s int) string {
	if s >= 0 && s < len(TermStatenames) {
		if TermStatenames[s] != "" {
			return TermStatenames[s]
		}
	}
	return __yyfmt__.Sprintf("state-%v", s)
}

func TermErrorMessage(state, lookAhead int) string {
	const TOKSTART = 4

	if !TermErrorVerbose {
		return "syntax error"
	}
	res := "syntax error: unexpected " + TermTokname(lookAhead)

	// To match Bison, suggest at most four expected tokens.
	expected := make([]int, 0, 4)

	// Look for shiftable tokens.
	base := TermPact[state]
	for tok := TOKSTART; tok-1 < len(TermToknames); tok++ {
		if n := base + tok; n >= 0 && n < TermLast && TermChk[TermAct[n]] == tok {
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}
	}

	if TermDef[state] == -2 {
		i := 0
		for TermExca[i] != -1 || TermExca[i+1] != state {
			i += 2
		}

		// Look for tokens that we accept or reduce.
		for i += 2; TermExca[i] >= 0; i += 2 {
			tok := TermExca[i]
			if tok < TOKSTART || TermExca[i+1] == 0 {
				continue
			}
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}

		// If the default action is to accept or reduce, give up.
		if TermExca[i+1] != 0 {
			return res
		}
	}

	for i, tok := range expected {
		if i == 0 {
			res += ", expecting "
		} else {
			res += " or "
		}
		res += TermTokname(tok)
	}
	return res
}

func Termlex1(lex TermLexer, lval *TermSymType) (char, token int) {
	token = 0
	char = lex.Lex(lval)
	if char <= 0 {
		token = TermTok1[0]
		goto out
	}
	if char < len(TermTok1) {
		token = TermTok1[char]
		goto out
	}
	if char >= TermPrivate {
		if char < TermPrivate+len(TermTok2) {
			token = TermTok2[char-TermPrivate]
			goto out
		}
	}
	for i := 0; i < len(TermTok3); i += 2 {
		token = TermTok3[i+0]
		if token == char {
			token = TermTok3[i+1]
			goto out
		}
	}

out:
	if token == 0 {
		token = TermTok2[1] /* unknown char */
	}
	if TermDebug >= 3 {
		__yyfmt__.Printf("lex %s(%d)\n", TermTokname(token), uint(char))
	}
	return char, token
}

func TermParse(Termlex TermLexer) int {
	return TermNewParser().Parse(Termlex)
}

func (Termrcvr *TermParserImpl) Parse(Termlex TermLexer) int {
	var Termn int
	var Termlval TermSymType
	var TermVAL TermSymType
	var TermDollar []TermSymType
	TermS := make([]TermSymType, TermMaxDepth)

	Nerrs := 0   /* number of errors */
	Errflag := 0 /* error recovery flag */
	Termstate := 0
	Termchar := -1
	Termtoken := -1 // Termchar translated into internal numbering
	Termrcvr.state = func() int { return Termstate }
	Termrcvr.lookahead = func() int { return Termchar }
	defer func() {
		// Make sure we report no lookahead when not parsing.
		Termstate = -1
		Termchar = -1
		Termtoken = -1
	}()
	Termp := -1
	goto Termstack

ret0:
	return 0

ret1:
	return 1

Termstack:
	/* put a state and value onto the stack */
	if TermDebug >= 4 {
		__yyfmt__.Printf("char %v in %v\n", TermTokname(Termtoken), TermStatname(Termstate))
	}

	Termp++
	if Termp >= len(TermS) {
		nyys := make([]TermSymType, len(TermS)*2)
		copy(nyys, TermS)
		TermS = nyys
	}
	TermS[Termp] = TermVAL
	TermS[Termp].yys = Termstate

Termnewstate:
	Termn = TermPact[Termstate]
	if Termn <= TermFlag {
		goto Termdefault /* simple state */
	}
	if Termchar < 0 {
		Termchar, Termtoken = Termlex1(Termlex, &Termlval)
	}
	Termn += Termtoken
	if Termn < 0 || Termn >= TermLast {
		goto Termdefault
	}
	Termn = TermAct[Termn]
	if TermChk[Termn] == Termtoken { /* valid shift */
		Termchar = -1
		Termtoken = -1
		TermVAL = Termlval
		Termstate = Termn
		if Errflag > 0 {
			Errflag--
		}
		goto Termstack
	}

Termdefault:
	/* default state action */
	Termn = TermDef[Termstate]
	if Termn == -2 {
		if Termchar < 0 {
			Termchar, Termtoken = Termlex1(Termlex, &Termlval)
		}

		/* look through exception table */
		xi := 0
		for {
			if TermExca[xi+0] == -1 && TermExca[xi+1] == Termstate {
				break
			}
			xi += 2
		}
		for xi += 2; ; xi += 2 {
			Termn = TermExca[xi+0]
			if Termn < 0 || Termn == Termtoken {
				break
			}
		}
		Termn = TermExca[xi+1]
		if Termn < 0 {
			goto ret0
		}
	}
	if Termn == 0 {
		/* error ... attempt to resume parsing */
		switch Errflag {
		case 0: /* brand new error */
			Termlex.Error(TermErrorMessage(Termstate, Termtoken))
			Nerrs++
			if TermDebug >= 1 {
				__yyfmt__.Printf("%s", TermStatname(Termstate))
				__yyfmt__.Printf(" saw %s\n", TermTokname(Termtoken))
			}
			fallthrough

		case 1, 2: /* incompletely recovered error ... try again */
			Errflag = 3

			/* find a state where "error" is a legal shift action */
			for Termp >= 0 {
				Termn = TermPact[TermS[Termp].yys] + TermErrCode
				if Termn >= 0 && Termn < TermLast {
					Termstate = TermAct[Termn] /* simulate a shift of "error" */
					if TermChk[Termstate] == TermErrCode {
						goto Termstack
					}
				}

				/* the current p has no shift on "error", pop stack */
				if TermDebug >= 2 {
					__yyfmt__.Printf("error recovery pops state %d\n", TermS[Termp].yys)
				}
				Termp--
			}
			/* there is no state on the stack with an error shift ... abort */
			goto ret1

		case 3: /* no shift yet; clobber input char */
			if TermDebug >= 2 {
				__yyfmt__.Printf("error recovery discards %s\n", TermTokname(Termtoken))
			}
			if Termtoken == TermEofCode {
				goto ret1
			}
			Termchar = -1
			Termtoken = -1
			goto Termnewstate /* try again in the same state */
		}
	}

	/* reduction by production Termn */
	if TermDebug >= 2 {
		__yyfmt__.Printf("reduce %v in:\n\t%v\n", Termn, TermStatname(Termstate))
	}

	Termnt := Termn
	Termpt := Termp
	_ = Termpt // guard against "declared and not used"

	Termp -= TermR2[Termn]
	// Termp is now the index of $0. Perform the default action. Iff the
	// reduced production is ε, $1 is possibly out of range.
	if Termp+1 >= len(TermS) {
		nyys := make([]TermSymType, len(TermS)*2)
		copy(nyys, TermS)
		TermS = nyys
	}
	TermVAL = TermS[Termp+1]

	/* consult goto table to find next state */
	Termn = TermR1[Termn]
	Termg := TermPgo[Termn]
	Termj := Termg + TermS[Termp].yys + 1

	if Termj >= TermLast {
		Termstate = TermAct[Termg]
	} else {
		Termstate = TermAct[Termj]
		if TermChk[Termstate] != -Termn {
			Termstate = TermAct[Termg]
		}
	}
	// dummy call; replaced with literal code
	switch Termnt {

	case 1:
		TermDollar = TermS[Termpt-1 : Termpt+1]
		//line parser.y:38
		{
			query = TermDollar[1].terms
		}
	case 2:
		TermDollar = TermS[Termpt-1 : Termpt+1]
		//line parser.y:42
		{
			program = TermDollar[1].rules
		}
	case 3:
		TermDollar = TermS[Termpt-1 : Termpt+1]
		//line parser.y:50
		{
			TermVAL.rules = make(Prules, 1)
			TermVAL.rules[0] = TermDollar[1].rule
		}
	case 4:
		TermDollar = TermS[Termpt-2 : Termpt+1]
		//line parser.y:55
		{
			TermVAL.rules = append(TermDollar[1].rules, TermDollar[2].rule)
		}
	case 5:
		TermDollar = TermS[Termpt-2 : Termpt+1]
		//line parser.y:62
		{
			TermVAL.rule = NewPrule(TermDollar[1].term, nil)
		}
	case 6:
		TermDollar = TermS[Termpt-4 : Termpt+1]
		//line parser.y:66
		{
			TermVAL.rule = NewPrule(TermDollar[1].term, TermDollar[3].terms)
		}
	case 7:
		TermDollar = TermS[Termpt-3 : Termpt+1]
		//line parser.y:70
		{
			TermVAL.rule = NewPrule(nil, TermDollar[2].terms)
		}
	case 8:
		TermDollar = TermS[Termpt-4 : Termpt+1]
		//line parser.y:77
		{
			TermVAL.term = NewPterm(TermDollar[1].name, TermDollar[3].terms)
		}
	case 9:
		TermDollar = TermS[Termpt-1 : Termpt+1]
		//line parser.y:81
		{
			TermVAL.term = NewPterm(TermDollar[1].name, nil)
		}
	case 10:
		TermDollar = TermS[Termpt-1 : Termpt+1]
		//line parser.y:88
		{
			TermVAL.terms = make(Pterms, 1)
			TermVAL.terms[0] = TermDollar[1].term
		}
	case 11:
		TermDollar = TermS[Termpt-3 : Termpt+1]
		//line parser.y:93
		{
			TermVAL.terms = append(TermDollar[1].terms, TermDollar[3].term)
		}
	case 12:
		TermDollar = TermS[Termpt-3 : Termpt+1]
		//line parser.y:100
		{
			TermVAL.term = NewPtermOp(TermDollar[2].name, TermDollar[1].term, TermDollar[3].term)
		}
	case 13:
		TermDollar = TermS[Termpt-1 : Termpt+1]
		//line parser.y:104
		{
			TermVAL.term = TermDollar[1].term
		}
	}
	goto Termstack /* stack new state and value */
}
