%{

package main

import (
    "fmt"
    "unicode"
    "strings"
    "errors"
)

var program Prules = nil
var query   Pterms = nil

%}

%union{
    name  string
    rule  *Prule
    rules Prules
    term  *Pterm
    terms Pterms
}

%type <rule> rule 
%type <rules> rules
%type <term> pred arg
%type <terms> arglist

%token <name> NAME
%token IMPL
%token <name> TERMOP

%%

start:  
        arglist
        {
            query = $1
        }
|       rules
        {
            program = $1
        }
;


rules : 
        rule
        { 
            $$ = make(Prules,1)
            $$[0] = $1 
        }
|       rules rule
        {
            $$ = append($1, $2)
        }
;

rule :  
        arg '.' 
        { 
            $$ = NewPrule($1,nil)
        }
|       arg IMPL arglist '.' 
        { 
            $$ = NewPrule($1,$3)
        }
|       IMPL arglist '.' 
        { 
            $$ = NewPrule(nil,$2)
        }
;
    
pred :  
        NAME '(' arglist ')'
        {   
            $$ = NewPterm($1,$3)
        }
|       NAME 
        { 
            $$ = NewPterm($1,nil) 
        }
;

arglist:
        arg
        { 
            $$ = make(Pterms,1)
            $$[0] = $1 
        }
|       arglist ',' arg  
        { 
            $$ = append($1, $3)
        }
;

arg:
        pred TERMOP pred 
        { 
            $$ = NewPtermOp($2,$1,$3) 
        }
|       pred
        {
            $$ = $1
        }
;


%% /* start of programs */

type TermLex struct {
    s string
    pos int
}

func (l *TermLex) Lex(lval *TermSymType) int {
    s := l.s
    s = strings.TrimLeftFunc(s, unicode.IsSpace) 
    if s == "" {
        return 0
    }
    for (s[0]=='#' || s[0]=='%') {
        i := 1
        for (s[i]!='\n') {
            i++
            if i >= len(s) {
               return 0
            }
        }
        s = s[i+1:]
        s = strings.TrimLeftFunc(s, unicode.IsSpace)
        if s == "" {
            return 0
        }
    }
    switch s[0] {
    case '(', ')','.',',':
        break
    case '/':
        fallthrough
    case '=':
        {
            l.s = s[1:]
            lval.name = string(s[0])
            return TERMOP
        }
    case ':':
        if s[1] == '-' {
            l.s = s[2:]
            return IMPL
        }
    case '"':
        if i := strings.Index(s[1:], `"`); i >= 0 {
            lval.name = string(s[1:i+1])
            l.s = s[i+2:]
            return NAME 
        }
    default:
        {   i := 0
            r := rune(s[i])
            for (unicode.IsLetter(r) || unicode.IsNumber(r)) && !unicode.IsSpace(r){
                i++
                if i >= len(s) {
                    break
                }
                r = rune(s[i])
            }
            lval.name = string(s[:i])
            l.s = s[i:]
            return NAME
        }
    }
    l.s = s[1:]
    return int(s[0])
}

func (l *TermLex) Error(s string) {
    fmt.Printf("syntax error: %s %s\n", l.s , s)
}

func ClearParserResult() {
    program = nil
    query = nil
}

func ParseQuery(input string) (Pterms,error) {
    ClearParserResult()
    if TermParse(&TermLex{s: input}) != 0 || program != nil {
        return nil, errors.New("syntax error in Query")
    }
    return query, nil
}

func ParseTerm(input string) (*Pterm,error) {
    ClearParserResult()
    if TermParse(&TermLex{s: input+"."}) != 0 || query != nil {
        return nil, errors.New("syntax error in Term")
    }
    return program[0].Head, nil
}


func ParseRules(input string) (Prules,error) {
    ClearParserResult()
    if TermParse(&TermLex{s: input}) != 0 || query != nil {
        return nil, errors.New("syntax error in Rules")
    }
    return program, nil
}
