package main

import (
	"fmt"
)

type Prules []*Prule

type Prule struct {
	Head *Pterm
	Body Pterms
}

func NewPrule(head *Pterm, body Pterms) *Prule {
	return &Prule{Head: head, Body: body}
}

type Pterms []*Pterm

type Pterm struct {
	Name string
	Args Pterms
}

func NewPterm(name string, args Pterms) *Pterm {
	return &Pterm{Name: name, Args: args}
}

func NewPtermOp(name string, arg1 *Pterm, arg2 *Pterm) *Pterm {
	args := make(Pterms, 2)
	args[0] = arg1
	args[1] = arg2
	return NewPterm(name, args)
}

func (ts Pterms) String() string {
	if len(ts) == 0 {
		return ""
	}
	s := fmt.Sprintf("%s", ts[0])
	for _, t := range ts[1:] {
		s = fmt.Sprintf("%s,%s", s, t)
	}
	return s
}

func (r *Prule) String() string {
	if r.Body != nil && len(r.Body) > 0 {
		return fmt.Sprintf("%s :- %s.", r.Head, r.Body)
	}
	return fmt.Sprintf("%s.", r.Head)
}

func (t *Pterm) String() string {
	if len(t.Args) == 0 {
		return t.Name
	}
	s := fmt.Sprintf("%s(%s", t.Name, t.Args[0])
	for _, a := range t.Args[1:] {
		s = fmt.Sprintf("%s,%s", s, a)
	}
	return s + ")"
}
